#ifndef _CHAN_PROPERTY_H
#define _CHAN_PROPERTY_H

#include "Channel.h"
#include "atlbase.h"
#include <vector>
using namespace std;

inline BOOL GetPropertyStr(HANDLE _hChan, CString* _pStr, long _PropType, long _SubProp=-1)
{
	VARIANT a_Value= {VT_EMPTY};
	if (_hChan==NULL) 
		return FALSE;
	CChannel* pChan= (CChannel*)_hChan;
	HRESULT Result= pChan->GetProperty(_PropType, a_Value, _SubProp, 0);
	if (a_Value.vt!=VT_BSTR || Result!=S_OK) 
	{
		*_pStr= "";
		return FALSE;
	} 
	else 
	{
		*_pStr= a_Value.bstrVal;
		SysFreeString(a_Value.bstrVal);
		return TRUE;
	}
}
inline BOOL SetPropertyStr(HANDLE _hChan, CString _pStr, long _PropType, long _SubProp=-1)
{
	CComVariant a_Value(_pStr);
	if (_hChan==NULL) return FALSE;
	CChannel* pChan=(CChannel*) _hChan;
	if (pChan->SetProperty(_PropType, a_Value, _SubProp, 0)==S_OK) return TRUE;
	else return FALSE;
}

inline BOOL GetPropertyLong(HANDLE _hChan, long* _pVal, long _PropType, long _SubProp=-1)
{
	VARIANT a_Value={VT_EMPTY};
	if (_hChan==NULL) return FALSE;
	CChannel* pChan=(CChannel*) _hChan;
	HRESULT Result=pChan->GetProperty(_PropType, a_Value, _SubProp, 0);
	if (a_Value.vt==VT_EMPTY || Result!=S_OK) {
		*_pVal=0;
		return FALSE;
	} else {
		*_pVal=a_Value.lVal;
		return TRUE;
	}
}

inline BOOL SetPropertyLong(HANDLE _hChan, long _Val, long _PropType, long _SubProp=-1)
{
	VARIANT a_Value;
	a_Value.vt=VT_I4;
	a_Value.lVal=_Val;
	if (_hChan==NULL) 
		return FALSE;
	CChannel* pChan=(CChannel*) _hChan;
	if (pChan->SetProperty(_PropType, a_Value, _SubProp, 0)==S_OK) 
		return TRUE;
	else return FALSE;
}

inline BOOL GetPropertyDbl(HANDLE _hChan, double* _pVal, long _PropType, long _SubProp=-1)
{
	VARIANT a_Value= {VT_EMPTY};
	if (_hChan==NULL) 
		return FALSE;
	CChannel* pChan= (CChannel*) _hChan;
	HRESULT Result= pChan->GetProperty(_PropType, a_Value, _SubProp, 0);
	if (a_Value.vt==VT_EMPTY || Result!=S_OK)
	{
		*_pVal=0.0;
		return FALSE;
	}
	else
	{
		*_pVal=a_Value.dblVal;
		return TRUE;
	}
}

inline BOOL SetPropertyDbl(HANDLE _hChan, double _Val, long _PropType, long _SubProp=-1)
{
	VARIANT a_Value;
	a_Value.vt=VT_R8;
	a_Value.dblVal=_Val;
	if (_hChan==NULL) return FALSE;
	CChannel* pChan=(CChannel*) _hChan;
	if (pChan->SetProperty(_PropType, a_Value, _SubProp, 0)==S_OK) return TRUE;
	else return FALSE;
}

inline BOOL CheckMultipleProp(long _Prop, vector<HANDLE>* _pChannels)
{
	VARIANT a_Value={VT_EMPTY};
	ASSERT(_pChannels);	
	if (_pChannels==NULL) 
		return FALSE;
	if (_pChannels->size()==0) 
		return FALSE;
	CChannel* pChan= (CChannel*) (*_pChannels)[0];
	HRESULT Result= pChan->GetProperty(_Prop, a_Value, -1, 0);
	if (a_Value.vt==VT_EMPTY || Result!=S_OK) 
		return FALSE;
	CComVariant Var1st= a_Value;
	for (int iChan= 0; iChan<_pChannels->size(); iChan++)  
	{
		a_Value.vt= VT_EMPTY;
		pChan= (CChannel*) (*_pChannels)[iChan];
		Result= pChan->GetProperty(_Prop, a_Value, -1, 0);
		if (a_Value.vt==VT_EMPTY || Result!=S_OK) 
			return FALSE;
		CComVariant VarNext=a_Value;
		if (Var1st!=VarNext) 
			return FALSE;
	}
	return TRUE;
}

// ***        ***
// : 
//     _Prop -  
//     _SubProp -  ,      
//     _pvChannels - 
// :
//    _SubProp -  
//    VARIANT -  
// !    ,    _SubProp!
static VARIANT GetNextMultipleVariant(TProperty _Prop, int* _SubProp, vector<HANDLE>* _pvChannels)
{
	VARIANT First={VT_EMPTY}, Next;
	if (_pvChannels->size()==0) return First;

	while (true) { //     (        )
		//        _SubProp
		First.vt=VT_EMPTY;
		HRESULT Result=((CChannel*)(*_pvChannels)[0])->GetProperty(_Prop, First, *_SubProp, 0);
		if (First.vt==VT_EMPTY) {
			return First; //    
		}

		CComVariant oleFirst=First;
		if (First.vt==VT_BSTR) SysFreeString(First.bstrVal);

		//     
		BOOL bFound=TRUE;
		for (int iChan=1; iChan<_pvChannels->size(); iChan++) {
			Next.vt=VT_EMPTY;
			Result=((CChannel*)(*_pvChannels)[iChan])->GetProperty(_Prop, Next, *_SubProp, 0);
			if (Next.vt==VT_EMPTY) {
				//  .      
				//       
				bFound=FALSE;
				break;
			}
			CComVariant oleNext=Next;
			if (Next.vt==VT_BSTR) SysFreeString(Next.bstrVal);
			if (oleNext!=oleFirst) {
				bFound=FALSE;
				break;
			}
		}
		if (bFound) { //   .  .
			return First;
		}
		(*_SubProp)++;
	}
}


#endif // _CHAN_PROPERTY_H