// version.h: interface and implementation of the CVersion class.
//
/////////////////////////////////////////////////////////////////////////////////

#ifndef MERA_VER_H
#define MERA_VER_H

const int VERSION_ALPHA=0;
const int VERSION_BETTA=1;
const int VERSION_RELEASE=2;

typedef struct{
	unsigned w  :4;
	unsigned bn :8;
	unsigned bf :8;
	unsigned min:8;
	unsigned maj:4;
}VERSION_DWORD,*PVERSION_DWORD;


typedef struct tagMERA_VERSION
{
	long   lMajorVersion;
	long   lMinorVersion;
	long   lBugFixNumber;
	long   lBuildNumber;
	char   chLetter;
}MERA_VERSION,*PMERA_VERSION;

#ifdef __cplusplus

#include "stdio.h"
#ifdef VERSION_MFC_USAGE
#include "stobject.h"
#include "cfgstream.h"
#include "string"
using namespace std;

#endif

#ifndef STDSTRSIZE
#define STDSTRSIZE 255
#endif


#ifndef ULONG 
typedef unsigned long ULONG;
#endif


#ifdef VERSION_MFC_USAGE
struct mera_version : public tagMERA_VERSION, public CStObject<CCfgStream>
#else
class mera_version : public tagMERA_VERSION
#endif
{
public:

	#ifdef VERSION_MFC_USAGE
	mera_version() : CStObject<CCfgStream>()
	#else
	mera_version()
	#endif
	{
		lMajorVersion=0;
		lMinorVersion=0;
		lBugFixNumber=0;
		lBuildNumber=0;
		chLetter='?';
	}

	#ifdef VERSION_MFC_USAGE
	mera_version(ULONG a_ulVersion):CStObject<CCfgStream>()
	#else
	mera_version(ULONG a_ulVersion)
	#endif
	{
		FromI4(a_ulVersion);
	}
	#ifdef VERSION_MFC_USAGE
	mera_version(long a_lMajorVersion, long a_lMinorVersion, long a_lBugFixNumber, long a_lBuildNumber, char a_chLetter)
			:CStObject<CCfgStream>()
	#else
	mera_version(long a_lMajorVersion, long a_lMinorVersion, long a_lBugFixNumber, long a_lBuildNumber, char a_chLetter)
	#endif

	{
		lMajorVersion=a_lMajorVersion;
		lMinorVersion=a_lMinorVersion;
		lBugFixNumber=a_lBugFixNumber;
		lBuildNumber=a_lBuildNumber;
		chLetter=a_chLetter;
	}

#ifdef VERSION_MFC_USAGE
	virtual void Write(CCfgStream& stream)const
	{
		CStFrame<CCfgStream> frame(stream);
		frame.Open(FALSE);
		frame.Put("lMajorVersion",lMajorVersion);
		frame.Put("lMinorVersion",lMinorVersion);
		frame.Put("lBugFixNumber",lBugFixNumber);
		frame.Put("lBuildNumber",lBuildNumber);
		string strWord="?";
		strWord[0]=chLetter;
		frame.Put("strWord",strWord);
		frame.Close();

	};

	virtual void Read(CCfgStream& stream)
    {
      CStFrame<CCfgStream> frame(stream);
      frame.Open(TRUE);
      frame.Get("lMajorVersion",lMajorVersion);
      frame.Get("lMinorVersion",lMinorVersion);
      frame.Get("lBugFixNumber",lBugFixNumber);
      frame.Get("lBuildNumber",lBuildNumber);
      string strWord;
      frame.Get("strWord",strWord);
      if(strWord.size())
        chLetter=strWord[0];
      else
        chLetter=' ';
      frame.Close();
    };
#endif
	virtual ~mera_version(){};
	
	//   
	inline bool operator == (const mera_version& a_verTest) const
	{
		return Equal(&a_verTest);
	}

	inline bool operator < (const mera_version& a_verTest) const
	{
		return Before(&a_verTest);
	}

	inline bool operator > (const mera_version& a_verTest) const
	{
		if(*this < a_verTest || *this == a_verTest) {
			return false;
		}
		else {
			return true;
		}
	}

	inline bool operator <= (const mera_version& a_verTest) const
	{
		if(*this<a_verTest||*this==a_verTest)return true;
		return false;
	}

	inline bool operator>=(mera_version& a_verTest)const
	{
		if(*this>a_verTest||*this==a_verTest)return true;
		return false;
	}


	inline bool operator!=(mera_version& a_verTest)const
	{
		return !Equal(&a_verTest);
	}

	inline operator const char*()
	{
		return AsText();
	}

	inline operator ULONG()
	{
		return AsI4();
	}

	inline operator PMERA_VERSION()
	{
	  return this;
	}


    bool Before(const mera_version* a_verTest)const
    {
      if(lMajorVersion<a_verTest->lMajorVersion)return true;
      if(lMajorVersion>a_verTest->lMajorVersion)return false;

      if(lMinorVersion<a_verTest->lMinorVersion)return true;
      if(lMinorVersion>a_verTest->lMinorVersion)return false;

      if(lBugFixNumber<a_verTest->lBugFixNumber)return true;
      if(lBugFixNumber>a_verTest->lBugFixNumber)return false;

      if(lBuildNumber<a_verTest->lBuildNumber)return true;
      if(lBuildNumber>a_verTest->lBuildNumber)return false;
      if(chLetter<a_verTest->chLetter)return true;
      if(chLetter>a_verTest->chLetter)return false;

      return false;
    }
    bool Equal(const mera_version* a_verTest)const
    {
      if(lMajorVersion!=a_verTest->lMajorVersion)return false;
      if(lMinorVersion!=a_verTest->lMinorVersion)return false;
      if(lBugFixNumber!=a_verTest->lBugFixNumber)return false;
      if(lBuildNumber!=a_verTest->lBuildNumber)return false;
      if(chLetter!=a_verTest->chLetter)return false;

      return true;
    }

    const char* AsText()
    {
      m_pchVersion[0]='\0';

      if(lBuildNumber!=0){
        sprintf(m_pchVersion,"%d.%d.%d.%d%c",lMajorVersion,lMinorVersion,lBugFixNumber,lBuildNumber,chLetter);
      }
      else{
        if(lBugFixNumber!=0){
          sprintf(m_pchVersion,"%d.%d.%d%c",lMajorVersion,lMinorVersion,lBugFixNumber,chLetter);
        }
        else{
          sprintf(m_pchVersion,"%d.%d%c",lMajorVersion,lMinorVersion,chLetter);
        }
      }
      return m_pchVersion;
    }

  
    ULONG AsI4()const
    {
      VERSION_DWORD vdw;
      vdw.maj=lMajorVersion;
      vdw.min=lMinorVersion;
      vdw.bf=lBugFixNumber;
      vdw.bn=lBuildNumber;
      if(chLetter=='a')vdw.w=VERSION_ALPHA;
      if(chLetter=='b')vdw.w=VERSION_BETTA;
      if(chLetter==' ')vdw.w=VERSION_RELEASE;

      return *((ULONG*)&vdw);
    }

    void FromI4(ULONG a_ulVersion)
    {
      VERSION_DWORD* vdw=(PVERSION_DWORD)&a_ulVersion;
      lMajorVersion=vdw->maj;
      lMinorVersion=vdw->min;
      lBugFixNumber=vdw->bf;
      lBuildNumber=vdw->bn;
      switch(vdw->w){
        case VERSION_ALPHA:
          chLetter='a';
        break;
        case VERSION_BETTA:
          chLetter='b';
        break;
        case VERSION_RELEASE:
          chLetter=' ';
        break;
        default:
          chLetter='?';
      }
    }
  private:
    char m_pchVersion[STDSTRSIZE];
};

#endif // __cplusplus
#endif
