#ifndef IOCONTROLIDS_H
#define IOCONTROLIDS_H

#include "mebase.h"
#include <static_assert.h>

#include "FIX_MEMMORY_ALIGN_BEGIN.h"

namespace mebius{ namespace ioctl{
typedef mebase::ULONG IOCTL_CODE;

const int UNLIMIT_SIZE        = 1;
const int MAX_ADDRESS_SIZE    = 128;
const int MAX_BITS_FIELD_SIZE = MAX_ADDRESS_SIZE;

enum{
	TYPEIO_MEAS_TASK         = 0x00001,
	TYPEIO_CALL_COMMAND      = 0x00002,
	TYPEIO_DATA              = 0x00003,
	TYPEIO_CYCLOGRAM_COMMAND = 0x00004
};

enum{
	IOCTL_BASE = 0,
	IOCTL_BASE_MR = 1
};

#define CTL_CODE( Type, Function, Method, Access ) (                 \
    ((Type) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) \
)

using mebase::TASKID;

///    
///       
const int MEASUREMENT_TASK_CLASSID = CLASSID_BASE_TASKS + 1;

///     
///       
const int DATA_TRANMITER_TASK_CLASSID = CLASSID_BASE_TASKS + 2;

///    
///       
const int CYCLOGRAM_TASK_CLASSID = CLASSID_BASE_TASKS + 3;

enum {
	MEASUREMENT_TASK_ID	= MAKE_TASKID(0, MEASUREMENT_TASK_CLASSID, 1, 0),
	DATA_TRANSMIT_TASK_ID	= MAKE_TASKID(0, DATA_TRANMITER_TASK_CLASSID, 1, 0),
 	CYCLOGRAM_TASK_ID	= MAKE_TASKID(0, CYCLOGRAM_TASK_CLASSID, 1, 0)
};


enum{
	///     
	IOCTL_MEASTASK_NULL                = CTL_CODE(TYPEIO_MEAS_TASK, 0x0000, 0, 0),
	IOCTL_MEASTASK_QUERY_SESSION_STATE = CTL_CODE(TYPEIO_MEAS_TASK, 0x0001, 0, 0),
	IOCTL_MEASTASK_DO_CLEANUP          = CTL_CODE(TYPEIO_MEAS_TASK, 0x0002, 0, 0),
	IOCTL_MEASTASK_CREATE_DEVICE       = CTL_CODE(TYPEIO_MEAS_TASK, 0x0003, 0, 0),
	
	IOCTL_MEASTASK_INIT_DEVICE         = CTL_CODE(TYPEIO_MEAS_TASK, 0x0004, 0, 0),
	IOCTL_MEASTASK_CONNECT_DEVICE      = CTL_CODE(TYPEIO_MEAS_TASK, 0x0005, 0, 0),
	IOCTL_MEASTASK_PROGRAMM_DEVICE     = CTL_CODE(TYPEIO_MEAS_TASK, 0x0006, 0, 0),
	
	IOCTL_MEASTASK_CREATE_CHAN         = CTL_CODE(TYPEIO_MEAS_TASK, 0x0007, 0, 0),
	IOCTL_MEASTASK_CREATE_CHANNELS     = CTL_CODE(TYPEIO_MEAS_TASK, 0x0008, 0, 0),
	IOCTL_MEASTASK_SET_SESSION_ID      = CTL_CODE(TYPEIO_MEAS_TASK, 0x0009, 0, 0),
	
	IOCTL_MEASTASK_PROGRAMM_DEVICE_BIN = CTL_CODE(TYPEIO_MEAS_TASK, 0x000A, 0, 0),
	
	IOCTL_MEASTASK_START               = CTL_CODE(TYPEIO_MEAS_TASK, 0x000B, 0, 0),
	IOCTL_MEASTASK_STOP                = CTL_CODE(TYPEIO_MEAS_TASK, 0x000C, 0, 0),

	IOCTL_MEASTASK_CALL_COMMAND        = CTL_CODE(TYPEIO_MEAS_TASK, 0x000D, 0, 0),
	///   . 
	///       
	IOCTL_MEASTASK_PROGRAM             = CTL_CODE(TYPEIO_MEAS_TASK, 0x0010, 0, 0),
	///  FTP 
	IOCTL_MEASTASK_START_FTP           = CTL_CODE(TYPEIO_MEAS_TASK, 0x0011, 0, 0),
	IOCTL_MEASTASK_ZBALANCE            = CTL_CODE(TYPEIO_MEAS_TASK, 0x0012, 0, 0),
	//  Bios
	IOCTL_MEASTASK_LOAD_BIOS           = CTL_CODE(TYPEIO_MEAS_TASK, 0x0013, 0, 0),
	IOCTL_MEASTASK_SET_NET_CONFIG      = CTL_CODE(TYPEIO_MEAS_TASK, 0x0014, 0, 0),
	IOCTL_MEASTASK_GET_NET_CONFIG      = CTL_CODE(TYPEIO_MEAS_TASK, 0x0015, 0, 0),
	///   i/o   
	IOCTL_MEASTASK_READ_RAM_RESOURCE   = CTL_CODE(TYPEIO_MEAS_TASK, 0x0016, 0, 0),
	///   i/o   
	IOCTL_MEASTASK_WRITE_RAM_RESOURCE  = CTL_CODE(TYPEIO_MEAS_TASK, 0x0017, 0, 0),
	///    i/o 
	IOCTL_MEASTASK_QUERY_RAM_RES_STATUS = CTL_CODE(TYPEIO_MEAS_TASK, 0x0018, 0, 0),
	///     
	IOCTL_CMD_SEARCH_DEVICE_INIT = CTL_CODE(TYPEIO_MEAS_TASK, 0x0019, 0, 0),
	///     
	IOCTL_CMD_GET_SEARCH_DEVICE_STATE = CTL_CODE(TYPEIO_MEAS_TASK, 0x001A, 0, 0),
	///    
	IOCTL_CMD_SEARCH_DEVICE_FINALIZE = CTL_CODE(TYPEIO_MEAS_TASK, 0x001B, 0, 0),
	
	///    
	IOCTL_MEASTASK_REQUEST_VERSION_INFO = CTL_CODE(TYPEIO_MEAS_TASK, 0x001C, 0, 0),
	///   
	IOCTL_MEASTASK_PROGRAM_VIRTCHANS = CTL_CODE(TYPEIO_MEAS_TASK, 0x001D, 0, 0)
};

enum{
	CMD_MEASTASK_TEST_NAND        = CTL_CODE(TYPEIO_MEAS_TASK, 0x0001, 0, 1),
	CMD_MEASTASK_TEST_SPI         = CTL_CODE(TYPEIO_MEAS_TASK, 0x0002, 0, 1),
	CMD_MEASTASK_MEASURE_CHAN     = CTL_CODE(TYPEIO_MEAS_TASK, 0x0003, 0, 1),
	CMD_MEASTASK_START_FTP_SERVER = CTL_CODE(TYPEIO_MEAS_TASK, 0x0004, 0, 1),
	CMD_MOTOR                     = CTL_CODE(TYPEIO_MEAS_TASK, 0x0005, 0, 1),
	CMD_ZBALANCE                  = CTL_CODE(TYPEIO_MEAS_TASK, 0x0006, 0, 1),
	CMD_READ_FLASH                = CTL_CODE(TYPEIO_MEAS_TASK, 0x0007, 0, 1),
	CMD_WRITE_FLASH               = CTL_CODE(TYPEIO_MEAS_TASK, 0x0008, 0, 1),
	CMD_MODE_MANIFOLD             = CTL_CODE(TYPEIO_MEAS_TASK, 0x0009, 0, 1),
	CMD_WRITE_DEVEEP              = CTL_CODE(TYPEIO_MEAS_TASK, 0x000A, 0, 1),
	CMD_READ_DEVEEP               = CTL_CODE(TYPEIO_MEAS_TASK, 0x000B, 0, 1),
	CMD_CLOSE_DEVEEP              = CTL_CODE(TYPEIO_MEAS_TASK, 0x000C, 0, 1),
	CMD_OPENREAD_DEVEEP           = CTL_CODE(TYPEIO_MEAS_TASK, 0x000D, 0, 1),
	CMD_OPENWRITE_DEVEEP          = CTL_CODE(TYPEIO_MEAS_TASK, 0x000E, 0, 1),
	CMD_GET_CALIBR_KOEF           = CTL_CODE(TYPEIO_MEAS_TASK, 0x000F, 0, 1),

	////  !!!   TYPEIO_MEAS_TASK   ...
	IOCTL_CMD_GET_CALIBR_KOEF     = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0001, 0, IOCTL_BASE),
	//     Omap
	IOCTL_CMD_GET_SOFT_VERSION    = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0002, 0, IOCTL_BASE),

	/*    
	 
	MIC-183:
	     (  )
	 ,      
	MIC-xxx:

	*/ 
	IOCTL_CMD_GET_ADC_VALUE       = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0003, 0, IOCTL_BASE),
	//       Omap
	IOCTL_CMD_LOAD_BLOCK_CALIBR   = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0004, 0, IOCTL_BASE),
	//      Flash
	IOCTL_CMD_WRITE_CALIBR_USER   = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0005, 0, IOCTL_BASE),
	//       Omap
	IOCTL_CMD_GET_BLOCK_CALIBR   = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0006, 0, IOCTL_BASE),
	//     flash 
	IOCTL_CMD_READ_CALIBR_USER   = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0007, 0, IOCTL_BASE),
	//   Flash     Omap
	IOCTL_CMD_RELOAD_CALIBR   = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0008, 0, IOCTL_BASE),
	//     ini   Flash
	IOCTL_CMD_WRITE_DEV_SETTINGS = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0009, 0, IOCTL_BASE),
	//     ini   Flash
	IOCTL_CMD_READ_DEV_SETTINGS = CTL_CODE(TYPEIO_CALL_COMMAND, 0x000A, 0, IOCTL_BASE),
	//    
	IOCTL_CMD_READ_CLB_MOD_TEMP = CTL_CODE(TYPEIO_CALL_COMMAND, 0x000B, 0, IOCTL_BASE),
	////   (   )     
	IOCTL_CMD_PUT_GAUGE           = CTL_CODE(TYPEIO_MEAS_TASK, 0x0010, 0, 1),
	///   
	IOCTL_CMD_MANIFOLD_POS= CTL_CODE(TYPEIO_MEAS_TASK, 0x0011, 0, 0),
	///     
	IOCTL_CMD_MANIFOLD_COUNTER= CTL_CODE(TYPEIO_MEAS_TASK, 0x0012, 0, 0),	
	///    
	IOCTL_CMD_GET_IN_RESIST = CTL_CODE(TYPEIO_MEAS_TASK, 0x0013, 0, 0),
	///   
	IOCTL_CMD_WRITE_AREG= CTL_CODE(TYPEIO_CALL_COMMAND, 0x0014, 0, IOCTL_BASE),
	///   
	IOCTL_CMD_READ_TEMP_SENSOR= CTL_CODE(TYPEIO_CALL_COMMAND, 0x0015, 0, IOCTL_BASE),
	////   
	IOCTL_CMD_PUT_MANIFOLD = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0016, 0, IOCTL_BASE),
	////   
	IOCTL_CMD_GET_MANIFOLD = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0017, 0, IOCTL_BASE),
	////   
	IOCTL_CMD_SAVE_MANIFOLD = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0018, 0, IOCTL_BASE),
	////   
	IOCTL_CMD_RESTORE_MANIFOLD = CTL_CODE(TYPEIO_CALL_COMMAND, 0x0019, 0, IOCTL_BASE), 
	/// ZBalance
	IOCTL_CMD_ZBALANCE_FINISH= CTL_CODE(TYPEIO_CALL_COMMAND, 0x001A, 0, IOCTL_BASE),
	IOCTL_CMD_ZBALANCE_GETSTATE= CTL_CODE(TYPEIO_CALL_COMMAND, 0x001B, 0, IOCTL_BASE),
	/// 
	IOCTL_CMD_INFO= CTL_CODE(TYPEIO_CALL_COMMAND, 0x001C, 0, IOCTL_BASE),
	///    
	IOCTL_CMD_CROSSBUS_CONVERT = CTL_CODE(TYPEIO_CALL_COMMAND, 0x001D, 0, IOCTL_BASE),
	
	///      
	///      
	IOCTL_CMD_READ_AUX_CHANS = CTL_CODE(TYPEIO_CALL_COMMAND, 0x001E, 0, IOCTL_BASE),
	
	/// 
	IOCTL_CMD_CYCLOGRAM_SETSTATE = CTL_CODE(TYPEIO_CYCLOGRAM_COMMAND, 0, 0, 0),
	IOCTL_CMD_CYCLOGRAM_STARTUPLOAD = CTL_CODE(TYPEIO_CYCLOGRAM_COMMAND, 0x0001, 0, 0),
	IOCTL_CMD_CYCLOGRAM_UPLOAD = CTL_CODE(TYPEIO_CYCLOGRAM_COMMAND, 0x0002, 0, 0),
	IOCTL_CMD_CYCLOGRAM_STOPUPLOAD = CTL_CODE(TYPEIO_CYCLOGRAM_COMMAND, 0x0003, 0, 0),
	IOCTL_CMD_CYCLOGRAM_GETPROPERTY = CTL_CODE(TYPEIO_CYCLOGRAM_COMMAND, 0x0004, 0, 0),
	IOCTL_CMD_CYCLOGRAM_STARTDEBUG = CTL_CODE(TYPEIO_CYCLOGRAM_COMMAND, 0x0005, 0, 0),
	IOCTL_CMD_CYCLOGRAM_STEPDEBUG = CTL_CODE(TYPEIO_CYCLOGRAM_COMMAND, 0x0006, 0, 0),
	IOCTL_CMD_CYCLOGRAM_STOPDEBUG = CTL_CODE(TYPEIO_CYCLOGRAM_COMMAND, 0x0007, 0, 0)
};

/// TYPEIO_DATA      
enum {
	///  
	IOCTL_DATA_PUT = CTL_CODE(TYPEIO_DATA, 0x0001, 0, 0),
	///  
	IOCTL_DATA_GET = CTL_CODE(TYPEIO_DATA, 0x0002, 0, 0)
};

typedef struct tagMEB_IOCTL_CREATE_OBJ_RESULT
{
	mebase::MEBRESULT mres;
	mebase::ULONG handle;
}MEB_IOCTL_CREATE_OBJ_RESULT;



typedef struct tagMEB_IOCTL_BITS_SET
{
	///   
	mebase::ULONG size;
	mebase::BYTE bits[MAX_BITS_FIELD_SIZE];
}MEB_IOCTL_BITS_SET;


typedef struct tagMEB_IOCTL_PROGRAMMING_BLOCK
{
	mebase::ULONG handle;
	mebase::ULONG size;
	mebase::BYTE buffer[1];
}MEB_IOCTL_PROGRAMMING_BLOCK;

typedef struct tagMEB_IOCTL_ADDRESS
{
	char address[MAX_ADDRESS_SIZE];
}MEB_IOCTL_ADDRESS;


typedef struct tagMEB_IOCTL_NET_CONFIG
{
	mebase::ULONG ip;
	mebase::ULONG mask;
	mebase::ULONG port;
	mebase::ULONG gateway;
	union {
		mebase::BYTE bMAC[8];
		mebase::ULONGLONG wMAC;
	};

}MEB_IOCTL_NET_CONFIG;


typedef struct tagMEB_IOCTL_DEVICE_INFO
{
	mebase::DEVICECLASSID devid;
	mebase::BUSID busid;
	char address[MAX_ADDRESS_SIZE];
}MEB_IOCTL_DEVICE_INFO;


///
/// mode 0 = read,
/// mode 1 = write
///
typedef struct tagMEB_IOCTL_RND_IO_BLOCK
{
	mebase::ULONG mode;
	mebase::ULONG id;
	mebase::ULONG status;
	mebase::ULONGLONG pos;
	mebase::ULONG size;
	mebase::BYTE data[1];
}MEB_IOCTL_RND_IO_BLOCK;

const int RND_IO_BLOCK_HEADER_SIZE = sizeof(mebase::ULONG) * 5 + sizeof(mebase::ULONGLONG);

#ifndef __gnu_linux__
STATIC_ASSERT((RND_IO_BLOCK_HEADER_SIZE + 4) == sizeof (MEB_IOCTL_RND_IO_BLOCK) );
#endif


///     IOCTL_MEASTASK_QUERY_RAM_RES_STATUS
typedef struct tagMEB_IOCTL_RND_IO_INFO
{
	mebase::ULONG id;
	mebase::ULONG status;
	mebase::ULONGLONG capacity;

}MEB_IOCTL_RND_IO_INFO;



typedef struct tagMEB_IOCTL_SESSION_STATE
{
	mebase::ULONG version;
	mebase::ULONG session_id;
	mebase::ULONG state;
}MEB_IOCTL_SESSION_STATE;

typedef struct tagMEB_IOCTL_SEARCH
{
	mebase::ULONG dev_count_;
	mebase::BYTE data_[UNLIMIT_SIZE];
} MEB_IOCTL_SEARCH;

typedef struct tagMEB_IOCTL_SEARCH_DEVICE_INFO
{
	mebase::ULONG size_;
	mebase::ULONG device_type_;
	mebase::ULONG serial_;
	mebase::BYTE addr_[UNLIMIT_SIZE];
} MEB_IOCTL_SEARCH_DEVICE_INFO;

const mebase::ULONG MEB_IOCTL_SEARCH_DEVICE_INFO_SIZE = 3 * sizeof(mebase::ULONG);


const int MODULE_VERSION_INFO_ID_LENGTH = 16;

///    .
///    .
typedef struct tagMEB_IOCTL_MODULE_VERSION_INFO
{
	///   
	mebase::BYTE id_[MODULE_VERSION_INFO_ID_LENGTH];

	///  ,  mera_ver
	mebase::ULONG ver_;

	///    time_t (32)
	mebase::ULONG timestamp_; 
	
}MEB_IOCTL_MODULE_VERSION_INFO;


const int VERSION_INFO_MODULES_COUNT = 8;

typedef struct tagMEB_IOCTL_FULL_VERSION_INFO
{
	///     ( ),  mera_ver
	mebase::ULONG hard_ctl_ver_;

	///     ,  mera_ver
	mebase::ULONG soft_ctl_ver_;
	
	///   , time_t (32)
	mebase::ULONG soft_ctl_timestamp_; 
	
	MEB_IOCTL_MODULE_VERSION_INFO module_[VERSION_INFO_MODULES_COUNT];
	
} MEB_IOCTL_FULL_VERSION_INFO;

const mebase::SHORT CYCLOGRAM_BUF_SIZE = 200;

typedef struct tagMEB_IOCTL_CYCLOGRAMM_CONTEXT {
	mebase::USHORT size_;
	mebase::BYTE context_[CYCLOGRAM_BUF_SIZE];
}MEB_IOCTL_CYCLOGRAMM_CONTEXT;


///   IOCTL 
///    ,    
///  ,       ,   
/// @bug      ,    ...
const 	mebase::SHORT MEB_IOCTL_COMMAND_SIGNATURE = (mebase::SHORT)0xF10C;

typedef struct tagMEB_IOCTL_COMMAND
{
	mebase::SHORT signature_;
	mebase::SHORT reserv_s_;	
	IOCTL_CODE code_;
	union{
		mebase::ULONG param_;
		mebase::MEBRESULT mres_;
		MEB_IOCTL_DEVICE_INFO dev_inf_;
		MEB_IOCTL_SESSION_STATE session_state_;
		MEB_IOCTL_CREATE_OBJ_RESULT obj_result_;
		MEB_IOCTL_ADDRESS address_;
		MEB_IOCTL_BITS_SET bits_mask_;
		MEB_IOCTL_PROGRAMMING_BLOCK progr_block_;
		MEB_IOCTL_NET_CONFIG net_config_;
		MEB_IOCTL_RND_IO_BLOCK ram_io_block_;
		MEB_IOCTL_RND_IO_INFO ram_io_status_;
		MEB_IOCTL_SEARCH search_inf_;
		MEB_IOCTL_FULL_VERSION_INFO version_inf_;
		MEB_IOCTL_CYCLOGRAMM_CONTEXT cyclogram_;

		mebase::BYTE reserved_[256];
	};
}MEB_IOCTL_COMMAND;

const size_t MEB_IOCTL_COMMAND_HEADER_SIZE = sizeof(IOCTL_CODE) + sizeof(mebase::SHORT) + sizeof(mebase::SHORT);

//sizeof(MEB_IOCTL_COMMAND::reserved_)
STATIC_ASSERT(sizeof(MEB_IOCTL_COMMAND) == (MEB_IOCTL_COMMAND_HEADER_SIZE + 256 ));


}}

#include "FIX_MEMMORY_ALIGN_END.h"
#endif
