#ifndef IDEVAPI_H
#define IDEVAPI_H

#include "mebase.h"
#include "IDevApiRegistry.h"
#include "IDevice.h"
#include "IItemsList.h"
#include "IBusList.h"
#include "IBus.h"
#include "IChansList.h"
#include "IDeviceDriverList.h"
#include "Include/IMebiusScale.h"
#include "IChangeDeviceStateCallback.h"


//    VC
#ifdef _MSC_VER
#include "MebiusDAQDef.h"
#else
#define MEBIUS_DAQ_API_DLL
#endif


namespace mebius_daq{

class IBus;
class IBusList;
class IMebiusDaqRegistry;
class IDeviceDriverList;
/// @interface DevAPI
/// Run Time (DART)  DevAPI Run Time (DART)
class IMebiusDaq : public IObject
{
public:
	virtual MEBRESULT MEBCALLTYPE Init(IDeviceDriver** pDriversList = 0) = 0;
	virtual MEBRESULT MEBCALLTYPE StartUp()=0;

	///  DART     .
	///     ,   .
	///      ,   .. , 
	///    
	virtual MEBRESULT MEBCALLTYPE Reset()=0;
	
	virtual MEBRESULT MEBCALLTYPE Uninit()=0;
	
	virtual MEBRESULT MEBCALLTYPE GetChannelsList(IChansList** ppList, const mebase::ULONG nFlags) = 0;
	virtual MEBRESULT MEBCALLTYPE GetDriversRegistry(IMebiusDaqRegistry** pRegistry) = 0;	


	virtual MEBRESULT MEBCALLTYPE GetBusList(IBusList** ppList) = 0;
	virtual MEBRESULT MEBCALLTYPE GetBus(IBus** pBus, const mebase::ULONG id) = 0;

	virtual MEBRESULT MEBCALLTYPE GetBusByAddr(IBus** pBus, const mebase::system::mestring& addr) = 0;
	virtual MEBRESULT MEBCALLTYPE GetDeviceByAddr(IDevice** pHostDevice, const mebase::system::mestring& addr) = 0;	


	///    
	///  
	virtual MEBRESULT MEBCALLTYPE GetDeviceDriversList(IBus* pBus, IDeviceDriverList** pList) = 0;

	///     IDeviceInfo.<br>
	///      ,    
	///  IBus::CreateDeviceLink
	virtual MEBRESULT MEBCALLTYPE CreateDeviceInfo(
				IDeviceInfo** ppInfo, 
				mebase::LPCSTR pchAddress, 
				mebase::ULONG nDevType, 
				IBus* pBus
			) = 0;

	///   IDeviceInfo.<br>
	virtual MEBRESULT MEBCALLTYPE DestroyDeviceInfo(
				IDeviceInfo* pInfo	
			) = 0;

	virtual MEBRESULT MEBCALLTYPE CreateDevice(IDevice** ppDevice, IDeviceInfo* ppInfo, IChangeDeviceStateCallback* a_pCallBack) = 0;
	
	virtual MEBRESULT MEBCALLTYPE DestroyDevice(IDevice* ppDevice) = 0;

	///      
	virtual MEBRESULT MEBCALLTYPE CreateScale(mebase::system::IMebiusScale** ppScale, mebase::ULONG nType) = 0;
	
	///   
	virtual MEBRESULT MEBCALLTYPE DestroyScale(mebase::system::IMebiusScale* pScale) = 0;

	///       
	virtual MEBRESULT MEBCALLTYPE SetCalibrDir(mebase::system::mestring& msPath) = 0;
	
	///       
	virtual MEBRESULT MEBCALLTYPE GetCalibrDir(mebase::system::mestring& msPath) const = 0;

	///      Bios
	virtual MEBRESULT MEBCALLTYPE SetBiosDir(mebase::system::mestring& msPath) = 0;
	
	///      Bios
	virtual MEBRESULT MEBCALLTYPE GetBiosDir(mebase::system::mestring& msPath) const = 0;

	///      
	//{TOFIX} FindChannelByAddress
	virtual MEBRESULT MEBCALLTYPE FindChannelByAdress(const mebase::system::mestring& msAddress, IChannel** ppChannel) = 0;
	
	///     .
	///          
	
	///  S_MEB_TRUE    
	/// S_MEB_FALSE   
	/// E_MEB_INVALIDARG    
	virtual MEBRESULT MEBCALLTYPE CompareDevices(const IDevice* pA, const IDevice* pB) const = 0;
	
	///         
	///      
	///    - ,   S_MEB_OK
	///      S_MEB_FALSE
	/// E_MEB_XXX   
	virtual MEBRESULT MEBCALLTYPE ProgramDevices() = 0;
	
	///          
	///  S_MEB_TRUE    
	/// S_MEB_FALSE   
	/// E_MEB_INVALIDARG    
	virtual MEBRESULT MEBCALLTYPE CompareChannels(const IChannel* pA, const IChannel* pB) const = 0;

};

///      DARTC:IMebiusDaq
const mebase::ULONG MEBLOCIID_XMebiusDaqEvents = 0xF0000001;

class XMebiusDaqEvents: public IObject
{
public:
	virtual MEBRESULT MEBCALLTYPE OnChangeChansList() = 0;
};

typedef IMebiusDaq IDevApi;
typedef IMebiusDaqRegistry IDevApiDaqRegistry;

///     DevApi Run Time (DART)
MEBIUS_DAQ_API_DLL MEBRESULT dartcCreate(IMebiusDaq ** ppDevApi);

///     DevApi Run Time (DART)
MEBIUS_DAQ_API_DLL MEBRESULT dartcDestroy(IMebiusDaq * pDevApi);

}

#endif
