#ifndef MEVARIANT_H
#define MEVARIANT_H

#include <string>
#include <limits>
#include "mebase.h"
#include "mestring.h"

///   " ..."
///      
/// MVT_OBJLIST   = 0x1000, mebius_daq::IValueList* pValues;
/// MVT_ITEMSLIST = 0x1001, mebius_daq::IItemsList* pItems;
///    mebius_daq::IObject* pObjVal;
///      mebius_daq,   

///   
///  MVT_VALLIST = 0x1000
///  mebius_daq::IValueList  forward, 
///       include,     -
namespace mebius_daq {
class IValueList;
}

namespace mebase{ namespace system{

enum MEVARENUM
{	
	MVT_EMPTY    = 0,
	MVT_NULL     = 1,
	MVT_I2       = 2,
	MVT_I4       = 3,
	MVT_LONG     = MVT_I4,
	MVT_INT      = MVT_I4,
	MVT_R4       = 4,
	MVT_R8       = 5,
	MVT_DOUBLE   = MVT_R8,
	MVT_BOOL     = 11,
	MVT_OBJECT   = 13,
	MVT_PTR      = MVT_OBJECT,
	MVT_I1       = 16,
	MVT_UI1      = 17,
	MVT_UI2      = 18,
	MVT_UI4      = 19,
	MVT_I8       = 20,
	MVT_UI8      = 21,
	MVT_STRING   = 73,
	MVT_VALLIST  = 0x1000,
	MVT_RESERVED  = 0x8000,
	MVT_PAIR      = MVT_RESERVED + 0x200
};

typedef ULONG MEVARTYPE;

template <class T>
inline MEVARTYPE mvt_by_type(T)      { assert(false); return MVT_EMPTY;}
inline MEVARTYPE mvt_by_type(char)   { return MVT_I1;}
inline MEVARTYPE mvt_by_type(BYTE)   { return MVT_UI1;}
inline MEVARTYPE mvt_by_type(SHORT)  { return MVT_I2;}
inline MEVARTYPE mvt_by_type(USHORT) { return MVT_UI2;}
inline MEVARTYPE mvt_by_type(LONG)   { return MVT_I4;}
inline MEVARTYPE mvt_by_type(ULONG)  { return MVT_UI4;}
inline MEVARTYPE mvt_by_type(LONGLONG)   { return MVT_I8;}
inline MEVARTYPE mvt_by_type(ULONGLONG)  { return MVT_UI8;}
inline MEVARTYPE mvt_by_type(float)   { return MVT_R4;}
inline MEVARTYPE mvt_by_type(double)  { return MVT_R8;}
inline MEVARTYPE mvt_by_type(bool)    { return MVT_BOOL;}


class CMeVariant
{
public:
	CMeVariant()
		:vt(MVT_EMPTY)
	{}

	CMeVariant(LONG val)
		:vt(MVT_LONG),
		lVal(val)
	{}
	
	CMeVariant(ULONGLONG val)
		:vt(MVT_UI8),
		ullVal(val)

	{}
	
	CMeVariant(SHORT val)
		:vt(MVT_I2),
		iVal(val)

	{}

	CMeVariant(const char* val)
		:vt(MVT_STRING),
		strVal(val)

	{}

	CMeVariant(const std::string& val)
		:vt(MVT_STRING),
		strVal(val)
		
	{}

	CMeVariant(float val)
		:vt(MVT_R4),
		fltVal(val)

	{}

	CMeVariant(double val)
		:dblVal(val),
		vt(MVT_R8)
	{}

	CMeVariant(bool val)
		:vt(MVT_BOOL),
		boolVal(val)
	{}

	CMeVariant(USHORT val)
		:vt(MVT_UI2),
		uiVal(val)
		
	{}

	CMeVariant(ULONG val)
		:vt(MVT_UI4),
		ulVal(val)
		
	{}

	CMeVariant(void* val)
		:vt(MVT_PTR),
		ptrVal(val)
		
	{}

	CMeVariant(char val)
		:cVal(val),
		vt(MVT_I1)
	{}

	CMeVariant(BYTE val)
		:bVal(val),
		vt(MVT_UI1)
	{}

	CMeVariant(LONGLONG val)
		:llVal(val),
		vt(MVT_I8)
	{}

	CMeVariant(mebius_daq::IValueList *val)
		:ptrValList(val),
		vt(MVT_VALLIST)
	{}

	CMeVariant(const CMeVariant& val)
	{
		internal_copy(val);
	}

	const CMeVariant& operator = (const CMeVariant& val)
	{
		internal_copy(val);
		return *this;
	}

	~CMeVariant()
	{}

	const MEVARTYPE GetType() const
	{
		return vt;
	}

	inline void* const GetObject() const
	{
		if (vt == MVT_PTR) {
			return ptrVal;
		}
		else {
			return 0;
		}
	}

	inline void* const GetPtrVal() const
	{
		if (vt == MVT_PTR) {
			return ptrVal;
		}
		else {
			return 0;
		}
	}

	inline const std::string GetString() const
	{
		if (vt == MVT_STRING) {
			return std::string(strVal);
		}
		else{
			return std::string();
		}
	}

	const std::string GetAsString() const
	{
		if (vt == MVT_STRING)
			return GetString();
		else switch (vt) {
			case MVT_BOOL:
				return (boolVal) ? "true" : "false";
			case MVT_I1:
				return NumberToString<SHORT>(cVal);
			case MVT_UI1:
				return NumberToString<USHORT>(bVal);
			case MVT_I2:
				return NumberToString<SHORT>(iVal);
			case MVT_UI2:
				return NumberToString<USHORT>(uiVal);
			case MVT_I4:
				return NumberToString<LONG>(lVal);
			case MVT_UI4:
				return NumberToString<ULONG>(ulVal);
			case MVT_R4:
				return NumberToString<float>(fltVal);
			case MVT_R8:
				return NumberToString<double>(dblVal);
			case MVT_I8:
				return NumberToString<LONGLONG>(ullVal);
			case MVT_UI8:
				return NumberToString<ULONGLONG>(ullVal);
			default:
				break;
		}
		return std::string();
	}

	mebius_daq::IValueList * GetValueList () const
	{
		if (vt == MVT_VALLIST) {
			return ptrValList;
		}
		else {
			return 0;
		}
	}
	
	///     
	template<class T>
	const T GetValue() const
	{
		if (vt == mvt_by_type(T())) {
			return *(T*)(&lVal);
		}
		else {
			assert(false);
			return T();
		}
	}

	operator bool() const { return GetValueAs<bool>(); }
	operator BYTE() const { return GetValueAs<BYTE>(); }
	operator char() const { return GetValueAs<char>(); }
	operator SHORT() const { return GetValueAs<SHORT>(); }
	operator USHORT() const { return GetValueAs<USHORT>(); }
	operator LONG() const { return GetValueAs<LONG>(); }
	operator ULONG() const { return GetValueAs<ULONG>(); }
	operator LONGLONG() const { return GetValueAs<LONGLONG>(); }
	operator ULONGLONG() const { return GetValueAs<ULONGLONG>(); }
	operator float() const { return GetValueAs<float>(); }
	operator double() const { return GetValueAs<double>(); }
	operator void*() const 
	{ 
		if (vt == MVT_OBJECT) 
			return ptrVal; 
		else { 
			assert(false); 
			return NULL; 
		}
	}
	operator mestring() const { return GetAsString(); }

protected:
	MEVARTYPE vt;

	union {
		ULONG ulVal;
		USHORT uiVal;
		ULONG uintVal;
		float fltVal;
		SHORT iVal;
		double dblVal;
		LONGLONG llVal;
		ULONGLONG ullVal;
		BYTE bVal;
		LONG lVal;
		bool boolVal;
		char cVal;
		int intVal;
		void* ptrVal;
		mebius_daq::IValueList * ptrValList;
	};
	mestring strVal;

private:
	void internal_copy(const CMeVariant& val)
	{
		vt = val.vt;
		switch (vt) {
			case MVT_EMPTY:
			case MVT_BOOL:
			case MVT_I1:
			case MVT_UI1:
			case MVT_I2:
			case MVT_UI2:
			case MVT_I4:
			case MVT_UI4:
			case MVT_R4:
			case MVT_R8:
			case MVT_I8:
			case MVT_UI8:
				//         POD   
				ullVal = val.ullVal;
			break;

			case MVT_STRING:
				strVal = val.strVal;
			break;

			///    ,   -    ...
			///     -    UI4
			case MVT_OBJECT:
				ptrVal = val.ptrVal;

			case MVT_VALLIST:
				ptrValList = val.ptrValList;
			break;

		}
	}

	template<typename T> T GetValueAs() const 
	{
	};

	template<> bool GetValueAs<bool>() const 
	{
		if (vt == MVT_BOOL)
			return GetValue<bool>();
		else {
			bool res = false;
			switch (vt) {
				case MVT_I1:
					res = cVal != 0; break;
				case MVT_UI1:
					res = bVal != 0; break;
				case MVT_I2:
					res = iVal != 0; break;
				case MVT_UI2:
					res = uiVal != 0; break;
				case MVT_I4:
					res = lVal != 0; break;
				case MVT_UI4:
					res = ulVal != 0; break;
				case MVT_I8:
					res = llVal != 0; break;
				case MVT_UI8:
					res = ullVal != 0; break;
				case MVT_R4:
					res = fltVal <= std::numeric_limits<float>::epsilon(); break;
				case MVT_R8:
					res = dblVal <= std::numeric_limits<double>::epsilon(); break;
				case MVT_OBJECT:
					res = ptrVal != 0; break;
				case MVT_VALLIST:
					res = ptrValList != 0; break;
				default:
					assert(false); break;
			}
			return res;
		}

	}
	template<> char GetValueAs<char>() const 
	{
		if (vt == MVT_I1)
			return GetValue<char>();
		else {
			char res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1 : 0; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> BYTE GetValueAs<BYTE>() const 
	{
		if (vt == MVT_UI1)
			return GetValue<BYTE>();
		else {
			BYTE res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1 : 0; break;
				case MVT_I1:
					res = cVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> SHORT GetValueAs<SHORT>() const 
	{
		if (vt == MVT_I2)
			return GetValue<SHORT>();
		else {
			SHORT res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1 : 0; break;
				case MVT_UI2:
					res = uiVal; break;
				case MVT_I1:
					res = cVal; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> USHORT GetValueAs<USHORT>() const 
	{
		if (vt == MVT_UI2)
			return GetValue<USHORT>();
		else {
			USHORT res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1 : 0; break;
				case MVT_I2:
					res = iVal; break;
				case MVT_I1:
					res = cVal; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> LONG GetValueAs<LONG>() const 
	{
		if (vt == MVT_I4)
			return GetValue<LONG>();
		else {
			LONG res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1 : 0; break;
				case MVT_UI4:
					res = ulVal; break;
				case MVT_I2:
					res = iVal; break;
				case MVT_UI2:
					res = iVal; break;
				case MVT_I1:
					res = cVal; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> ULONG GetValueAs<ULONG>() const 
	{
		if (vt == MVT_UI4)
			return GetValue<ULONG>();
		else {
			ULONG res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1 : 0; break;
				case MVT_I4:
					res = lVal; break;
				case MVT_I2:
					res = iVal; break;
				case MVT_UI2:
					res = iVal; break;
				case MVT_I1:
					res = cVal; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> LONGLONG GetValueAs<LONGLONG>() const 
	{
		if (vt == MVT_I8)
			return GetValue<LONGLONG>();
		else {
			LONGLONG res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1 : 0; break;
				case MVT_UI8:
					res = ullVal; break;
				case MVT_I4:
					res = lVal; break;
				case MVT_UI4:
					res = ulVal; break;
				case MVT_I2:
					res = iVal; break;
				case MVT_UI2:
					res = iVal; break;
				case MVT_I1:
					res = cVal; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> ULONGLONG GetValueAs<ULONGLONG>() const 
	{
		if (vt == MVT_UI8)
			return GetValue<ULONGLONG>();
		else {
			ULONGLONG res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1 : 0; break;
				case MVT_I8:
					res = llVal; break;
				case MVT_I4:
					res = lVal; break;
				case MVT_UI4:
					res = ulVal; break;
				case MVT_I2:
					res = iVal; break;
				case MVT_UI2:
					res = iVal; break;
				case MVT_I1:
					res = cVal; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> float GetValueAs<float>() const 
	{
		if (vt == MVT_R4)
			return GetValue<float>();
		else {
			float res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? (float)1.0 : (float)0.0; break;
				case MVT_I2:
					res = iVal; break;
				case MVT_UI2:
					res = iVal; break;
				case MVT_I1:
					res = cVal; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
	template<> double GetValueAs<double>() const 
	{
		if (vt == MVT_R8)
			return GetValue<double>();
		else {
			double res = 0;
			switch (vt) {
				case MVT_BOOL:
					res = boolVal ? 1.0 : 0.0; break;
				case MVT_I4:
					res = lVal; break;
				case MVT_UI4:
					res = ulVal; break;
				case MVT_I2:
					res = iVal; break;
				case MVT_UI2:
					res = iVal; break;
				case MVT_I1:
					res = cVal; break;
				case MVT_UI1:
					res = bVal; break;
				default:
					assert(false); break;
			}
			return res;
		}
	}
};


}}
#endif
