#ifndef IPROCESSING_H
#define IPROCESSING_H
#include "itag.h"
#include "ISettingsStorage.h"


namespace medyan {


// {F40DB412-CE24-447d-9BC7-D39121529C6B}
extern const GUID IID_IProcessingSettings;


/// @interface IProcessingSettings
///  .        
/// 1.  QueryInterface   
/// 2.  Set / GetRowSettings     
///  
class IProcessingSettings : public IUnknown
{
public:
	///   
	virtual HRESULT STDMETHODCALLTYPE GetRawSettings(void* pRaw) const = 0;
	
	///   
	virtual HRESULT STDMETHODCALLTYPE SetRawSettings(void* pRaw) = 0;
	
	///   .
	///     146       
	virtual HRESULT STDMETHODCALLTYPE Clone(IProcessingSettings** ppClone) const = 0;
};


// {9D8529F3-D168-439a-9611-3B39A12DD1D4}
extern const GUID IID_IProcessing;

// {CCB9B93D-0246-48cc-B7D3-27CB44FAF014}
extern const GUID IID_IProcessingFactory;


typedef __int64 PROCESSINGID;

class IProcessing : public IUnknown
{
public:
	
	virtual HRESULT STDMETHODCALLTYPE GetGuid(GUID* pId) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetName(BSTR* pbstrName) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetInfo(BSTR* pbstrInfo) = 0;

	virtual HRESULT STDMETHODCALLTYPE Init(PROCESSINGID id) = 0;
	///    
	virtual HRESULT STDMETHODCALLTYPE Program() = 0;
	
	virtual HRESULT STDMETHODCALLTYPE Clear() = 0;
	virtual HRESULT STDMETHODCALLTYPE Start() = 0;
	virtual HRESULT STDMETHODCALLTYPE Stop() = 0;
	virtual HRESULT STDMETHODCALLTYPE OnUpdateData() = 0;

	///   
	///       (IProcessingLink),    
	///  
	virtual HRESULT STDMETHODCALLTYPE LinkInput(IUnknown* pLink) = 0;

	///   
	///       (IProcessingLink),    
	///  
	virtual HRESULT STDMETHODCALLTYPE LinkOutput(IUnknown* pLink) = 0;

	///   
	///    LinkOutput
	///      
	virtual HRESULT STDMETHODCALLTYPE CreateOutput(IUnknown** ppLink) = 0;

	virtual HRESULT STDMETHODCALLTYPE StoreSettings(ISettingsStorageOut* pStorage) const = 0;
	virtual HRESULT STDMETHODCALLTYPE RestoreSettings(ISettingsStorageIn* pStorage) = 0;

	virtual HRESULT STDMETHODCALLTYPE GetSettings(IProcessingSettings**) = 0;
	virtual HRESULT STDMETHODCALLTYPE SetSettings(IProcessingSettings*) = 0;
	
	virtual HRESULT STDMETHODCALLTYPE GetStoreSettingsFlag(VARIANT_BOOL* pFlag) const = 0;
	virtual HRESULT STDMETHODCALLTYPE SetStoreSettingsFlag(VARIANT_BOOL flag) = 0;
	

};


/// IProcessing::QueryInterface
class IProcessingLink  /// ==IProcessingOutput  ==IProcessingInput
{
	///     ....
	///    
	virtual HRESULT STDMETHODCALLTYPE ConnectTag(int nIndx, ITag* pTag) = 0;
	
	///   ,    ...
	virtual HRESULT GetCapacity(ULONG* nLong) = 0;
	
};

class IProcessingFactory : public IUnknown
{
public:
	virtual HRESULT STDMETHODCALLTYPE GetName(BSTR* pbstrName) = 0;
	virtual HRESULT STDMETHODCALLTYPE GetDescribe(BSTR* pbstrDesc) = 0;
	virtual HRESULT STDMETHODCALLTYPE CreateProcessing(IProcessing** ppProcessing) = 0;
};

}
#endif

